from django import forms
from django.contrib import admin
from django.contrib.auth.models import Group
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.core.exceptions import ValidationError

from .models import User


class UserCreationForm(forms.ModelForm):
    """A form for creating new users. Includes all the required
    fields, plus a repeated password."""

    password1 = forms.CharField(label="Password", widget=forms.PasswordInput)
    password2 = forms.CharField(
        label="Password confirmation", widget=forms.PasswordInput
    )

    class Meta:
        model = User
        fields = (
            "email",
            "first_name",
            "last_name",
            "is_active",
            "is_admin",
            "is_superuser",
        )

    def clean_password2(self):
        # Check that the two password entries match
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise ValidationError("Passwords don't match")
        return password2

    def save(self, commit=True):
        # Save the provided password in hashed format
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class UserChangeForm(forms.ModelForm):
    """A form for updating users. Includes all the fields on
    the user, but replaces the password field with admin's
    disabled password hash display field.
    """

    password = ReadOnlyPasswordHashField()

    class Meta:
        model = MyUser
        fields = fields = (
            "email",
            "password",
            "first_name",
            "last_name",
            "is_active",
            "is_admin",
            "is_superuser",
        )


class UserAdmin(BaseUserAdmin):
    # The forms to add and change user instances
    form = UserChangeForm
    add_form = UserCreationForm

    # The fields to be used in displaying the User model.
    # These override the definitions on the base UserAdmin
    # that reference specific fields on auth.User.
    list_display = ("email", "last_name", "is_admin", "created")
    list_filter = ("is_admin",)
    fieldsets = (
        (None, {"fields": ("email", "password")}),
        ("Personal info", {"fields": ("first_name", "last_name")}),
        ("Permissions", {"fields": ("is_active", "is_admin", "is_superuser")}),
    )
    # add_fieldsets is not a standard ModelAdmin attribute. UserAdmin
    # overrides get_fieldsets to use this attribute when creating a user.
    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": ("email", "date_of_birth", "password1", "password2"),
            },
        ),
    )
    search_fields = ("email",)
    ordering = ("email",)
    filter_horizontal = ()


# Now register the new UserAdmin...
admin.site.register(MyUser, UserAdmin)
# ... and, since we're not using Django's built-in permissions,
# unregister the Group model from admin.
admin.site.unregister(Group)



from django.conf.global_settings import AUTHENTICATION_BACKENDS
from django.contrib.auth import authenticate, login, logout
from django.http import HttpResponse
from django.views.decorators.csrf import ensure_csrf_cookie

from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from .emails import email_user_activation
from .models import User
from .serializers import AuthSerializer


@api_view(["POST"])
@permission_classes([])
def frontend_signup(request):
    """
    Sign up user

    Args:
        request:
            The request body should contain a JSON object such as::
                {
                    "first_name": "John",
                    "last_name": "Doe",
                    "email": "email@ex.com",
                    "password": "secret_pa$$w0rD",
                }

    Returns:
        JSON object::
            AuthSerializer
    """
    user = AuthSerializer(data=request.data)
    user.is_valid(raise_exception=True)
    user = user.save()
    email_user_activation(request, user)

    return Response(status=201)


@api_view(["POST"])
@permission_classes([])
def frontend_login(request):
    """
    Log in a user

    Args:
        request:
            The request body should contain a JSON object such as::

              {"email": "email@ex.com",
               "password": "secret_pa$$w0rD"}

    Returns:
        JSON object::
            AuthSerializer
    """

    data = request.data
    email, password = data["email"].lower(), data["password"]

    user = authenticate(email=email, password=password)

    if user is not None:
        user.backend = AUTHENTICATION_BACKENDS[0]
        login(request, user)
        return Response(AuthSerializer(user).data)
    else:
        return HttpResponse("Email et mot de passe ne correspondent pas", status=400)


@api_view(["POST"])
def frontend_logout(request):
    """Log out view."""
    logout(request)
    return HttpResponse(status=200)


@api_view(["GET"])
@ensure_csrf_cookie
@permission_classes([])
def who_am_i(request):
    """Returns information about the current user."""
    if request.user.is_anonymous:
        return Response(status=400)

    return Response(AuthSerializer(request.user).data)


@api_view(["POST"])
@permission_classes([])
def user_activation(request):
    """Active user"""
    try:
        user = User.objects.get(
            email=request.data.get("email"),
            activation_key=request.data.get("activation_key"),
        )
    except User.DoesNotExist:
        return Response(status=404)

    if not user.is_active:
        user.is_active = True
        user.save()

    return Response(status=200)



from django.core import exceptions
from django.contrib.auth.hashers import make_password
from django.contrib.auth import password_validation
from rest_framework import serializers
from .models import User


class AuthSerializer(serializers.ModelSerializer):
    first_name = serializers.CharField(required=True, max_length=150)
    last_name = serializers.CharField(required=True, max_length=150)
    email = serializers.EmailField(required=True, max_length=100)
    password = serializers.CharField(write_only=True, required=True, max_length=100)

    class Meta:
        model = User
        fields = (
            "first_name",
            "last_name",
            "email",
            "password",
            "is_superuser",
            "is_admin",
            "is_staff",
        )

    def validate_password(self, value):
        errors = None
        try:
            password_validation.validate_password(password=value, user=User)
        except exceptions.ValidationError as e:
            errors = {"password": list(e.messages)}

        if errors:
            raise serializers.ValidationError(errors)
        return make_password(value)



# Generated by Django 3.1 on 2020-10-28 17:55

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('email', models.EmailField(max_length=255, unique=True, verbose_name='email address')),
                ('first_name', models.CharField(default='', max_length=150)),
                ('last_name', models.CharField(default='', max_length=150)),
                ('is_active', models.BooleanField(default=False)),
                ('is_admin', models.BooleanField(default=False)),
                ('activation_key', models.UUIDField(default=uuid.uuid4, editable=False)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.contrib.auth.models import BaseUserManager
from typing import Optional


class UserManager(BaseUserManager):
    def create_user(
        self,
        email: str,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        password: Optional[str] = None,
        is_active: Optional[bool] = False,
        is_admin: Optional[bool] = False,
        is_superuser: Optional[bool] = False,
    ):
        """
        Creates and saves a User with the given email and password.
        """
        if not email:
            raise ValueError("Users must have an email address")

        user = self.model(
            email=self.normalize_email(email),
            first_name=first_name,
            last_name=last_name,
            is_active=is_active,
            is_admin=is_admin,
            is_superuser=is_superuser,
        )
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(
        self,
        email: str,
        first_name: Optional[str] = "toto",
        last_name: Optional[str] = "admin",
        password=None,
        is_active: bool = True,
    ):
        """
        Creates and saves a superuser with the given email and password.
        """
        user = self.create_user(
            email=email,
            first_name=first_name,
            last_name=last_name,
            password=password,
        )
        user.is_admin = True
        user.is_superuser = True
        user.is_active = is_active
        user.save(using=self._db)
        return user



import uuid

from django.db import models
from django.contrib.auth.models import AbstractBaseUser

from .managers import UserManager


class User(AbstractBaseUser):
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    username = None
    email = models.EmailField(
        verbose_name="email address",
        max_length=255,
        unique=True,
    )
    first_name = models.CharField(max_length=150, default="")
    last_name = models.CharField(max_length=150, default="")
    is_active = models.BooleanField(default=True)
    is_admin = models.BooleanField(default=False)
    is_superuser = models.BooleanField(default=False)
    activation_key = models.UUIDField(default=uuid.uuid4, editable=False)

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []

    objects = UserManager()

    def __str__(self):
        return self.email

    def has_perm(self, perm, obj=None):
        "Does the user have a specific permission?"
        # Simplest possible answer: Yes, always
        return True

    def has_module_perms(self, app_label):
        "Does the user have permissions to view the app `app_label`?"
        # Simplest possible answer: Yes, always
        return True

    @property
    def is_staff(self):
        "Is the user a member of staff?"
        # Simplest possible answer: All admins are staff
        return self.is_admin



# Generated by Django 4.0.2 on 2022-05-10 07:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('telescoop_auth', '0002_alter_user_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='is_superuser',
            field=models.BooleanField(default=False),
        ),
    ]



from django.urls import path

from . import views

app_name = "telescoop_auth"
urlpatterns = [
    path("login", views.frontend_login),
    path("logout", views.frontend_logout),
    path("profile", views.who_am_i, name="auth_profile"),
    path("signup", views.frontend_signup),
    path("user/activation", views.user_activation),
]



from smtplib import SMTPException

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string


def send_email(
    template_directory,
    file_name,
    subject,
    receiver,
    subject_id="",
    data=None,
    attachment=None,
):
    txt = render_to_string(f"{template_directory}/emails/txt/{file_name}.html", data)
    html = render_to_string(f"{template_directory}/emails/html/{file_name}.html", data)
    try:
        msg = EmailMultiAlternatives(
            subject=subject_id + subject,
            body=txt,
            to=receiver,
        )

        msg.attach_alternative(html, "text/html")
        msg.extra_headers["X-Mailgun-Tag"] = [file_name]
        if attachment:
            msg.attach(attachment["name"], attachment["content"], "application/pdf")
        msg.send()
    except SMTPException as e:
        print("There was an error sending an email: ", e)


def email_user_activation(request, user):
    data = {
        "title": "Vérifier votre email",
        "url": (
            f"{settings.FRONT_END_URL}/user/activation?email={user.email}"
            f"&activation_key={user.activation_key}"
        ),
    }

    send_email(
        template_directory="telescoop_auth",
        file_name="user_activation",
        subject=data["title"],
        data=data,
        receiver=[user.email],
    )



import json

from django.contrib.auth import get_user_model
from django.test import TestCase, Client
from django.urls import reverse

User = get_user_model()


class UserDetailsTestCase(TestCase):
    def setUp(self):
        User.objects.create_user(
            email='user@mail.com',
            first_name='first',
            last_name='last',
            password='password',
            is_active=True,
        )

    def test_user_details(self):
        user = User.objects.get(email='user@mail.com')
        client = Client()
        client.login(email=user.email, password='password')
        res = client.get(reverse("auth_profile"))
        response_data = json.loads(res.content)
        self.assertEqual(response_data['email'], 'user@mail.com')
        self.assertEqual(response_data['first_name'], 'first')
